package com.test;

import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;

/**
 * Servlet request wrapper to wrap the incoming multipart request
 * and provide access to the params etc in the same way as for normal requests.
 */
public class FileUploadRequestWrapper extends HttpServletRequestWrapper
{
    //Store the set of params and files retrieved from the request.
    private Hashtable params;
    private Hashtable files;


    public FileUploadRequestWrapper(HttpServletRequest request)
    {
        super(request);
        params = new Hashtable();
        files = new Hashtable();

        try
        {
            // Create a factory for disk-based file items
            FileItemFactory factory = new DiskFileItemFactory();

            // Create a new file upload handler
            ServletFileUpload upload = new ServletFileUpload(factory);

            // Parse the request
            List items = upload.parseRequest(request);

            // Process the uploaded items
            Iterator it = items.iterator();
            while (it.hasNext())
            {
                FileItem item = (FileItem) it.next();

                if (item.isFormField())
                {
                    if (params.containsKey(item.getFieldName()))
                    {
                        ArrayList values = (ArrayList) params.get(item.getFieldName());
                        values.add(item.getString());
                    }
                    else
                    {
                        ArrayList values = new ArrayList();
                        values.add(item.getString());
                        params.put(item.getFieldName(), values);
                    }
                }
                else
                {
                    files.put(item.getFieldName(), item);

                }
            }
        }
        catch (Exception e)
        {
            System.out.println("Error processing request! " + e.getMessage());
            e.printStackTrace();
        }

    }

    // Override the ServletRequest parameter access methods to
    // return the params from the multipart request

    public Enumeration getParameterNames()
    {
        return params.keys();
    }

    public String[] getParameterValues(String paramName)
    {
        if (params.get(paramName) != null)
            return (String[]) ((ArrayList) params.get(paramName)).toArray(new String[1]);
        else
            return null;
    }

    public String getParameter(String paramName)
    {
        if (params.get(paramName) != null)
            return (String) ((ArrayList) params.get(paramName)).get(0);
        else
            return null;
    }

    // add new methods for accessing the files contained in the
    // multipart request

    public Enumeration getFileNames()
    {
        return files.keys();
    }

    public FileItem getFile(String filename)
    {
        return (FileItem) files.get(filename);
    }
}
